-- holder for some lib functions
local cur_version=10
if _G["HED_LIB"] and hedlib.ver>=cur_version then 
	-- print ("Hed_lib "..cur_version.." is already")
	else 

hedlib = _G["HED_LIB"] or CreateFrame("Frame","HED_LIB",UIParent)
hedlib.ver=cur_version

--print("Hedd_lib version "..cur_version)

hedlib.dummy = function() end

hedlib.round = function (n)
	return floor(n+0.5)
end

hedlib.isPlayer = function(unit)
	if(unit == 'player' or unit == 'vehicle' or unit == 'pet') then
		return true
	else
		return nil
	end
end

hedlib.isEnemy = function(unit)
	if (UnitIsEnemy("player", unit) or UnitCanAttack("player", unit)) then
		return true
	else
		return nil
	end
end

hedlib.siValue = function(val)
	if val==0 then
		return
	elseif val >= 10000000 or val <= -10000000 then 
        return string.format("%.1fm", val / 1000000) 
    elseif val >= 1000000 or val <= -1000000 then
        return string.format("%.2fm", val / 1000000) 
    elseif val >= 100000 or val <= -100000 then
        return string.format("%.0fk", val / 1000) 
    elseif val >= 1000 or val <= -1000 then
        return string.format("%.1fk", val / 1000) 
    else
        return ceil(val)
    end
end

hedlib.hex = function(r, g, b)
	if(type(r) == 'table') then
		if(r.r) then r, g, b = r.r, r.g, r.b else r, g, b = unpack(r) end
	end
	return ('|cff%02x%02x%02x'):format(r * 255, g * 255, b * 255)
end

hedlib.fontsize = function(height)
	local FRAME_SIZE,FONT_SIZE = 1,0.9
	local size = FONT_SIZE * (height / FRAME_SIZE)
	size = (size>9) and size or 9
	return(hedlib.round(size))
end

hedlib.CreateBD = function(f,border,color, bgcolor)
	color = color or {1,0,0,1}
	border = border or 5
	local backdrop_tab = { 
    bgFile = hedlib.backdrop_texture, 
    edgeFile = hedlib.backdrop_edge_texture,
    tile = false,
    tileSize = 0, 
    edgeSize = border, 
    insets = { 
      left = border, 
      right = border, 
      top = border, 
      bottom = border,
    },
  	}
	f.bd = f.bd or CreateFrame("Frame", nil, f)
    f.bd:SetFrameLevel(0)
    f.bd:SetPoint("TOPLEFT",-border,border)
    f.bd:SetPoint("BOTTOMRIGHT",border,-border)
    f.bd:SetBackdrop(backdrop_tab)
	bgcolor = bgcolor or {0,0,0,0}
    f.bd:SetBackdropColor(unpack(bgcolor))
   	f.bd:SetBackdropBorderColor(unpack(color))
	return f.bd
end

hedlib.CreateBG=function(f,border,coltex,color)
	f.bg = f.bg or f:CreateTexture(nil, "BACKGROUND")
	border=border or 0
	if (border==0) then
		f.bg:SetAllPoints(f)
	else
		f.bg:SetPoint("TOPLEFT", -border, border)
		f.bg:SetPoint("BOTTOMRIGHT", border, -border)
	end
	coltex = coltex or {0,0,0}
	if type(coltex) == "string" then
		f.bg:SetTexture(coltex)
		if color then
			f.bg:SetVertexColor(unpack(color))
		end
	else	
		f.bg:SetTexture(unpack(coltex))
	end
	return f.bg
end

hedlib.CreateOverlay=function(f,border,coltex,color)
	f.overlay = f.overlay or f:CreateTexture(nil, "BACKGROUND")
	border=border or 0
	if (border==0) then
		f.overlay:SetAllPoints(f)
	else
		f.overlay:SetPoint("TOPLEFT", -border, border)
		f.overlay:SetPoint("BOTTOMRIGHT", border, -border)
	end
	coltex = coltex or {0,0,0}
	if type(coltex) == "string" then
		f.overlay:SetTexture(coltex)
		if color then
			f.overlay:SetVertexColor(unpack(color))
		end
	else	
		f.overlay:SetTexture(unpack(coltex))
	end
	return f.overlay
end

hedlib.CreateFont = function(f, font, size, outline)
	if size then 
		if size<1 then 
			size = hedlib.fontsize(f:GetHeight())+size
		end
	else
		size = hedlib.fontsize(f:GetHeight())
	end
	
	local fs = f:CreateFontString(nil, "OVERLAY")
	fs:SetFont(font, size, outline)
	fs:SetShadowColor(0,0,0,1)
	return fs
end  

hedlib.UpdateFS=function(f, size, justify)
    local fo,si,fl=f:GetFont()
    f:SetFont(fo, size, fl)
    f:SetShadowColor(0, 0, 0, 0)
    if(justify) then f:SetJustifyH(justify) end
    return f
end

hedlib.ColorGradient = function(perc, ...)
	if perc >= 1 then
		local r, g, b = select(select('#', ...) - 2, ...)
		return r, g, b
	elseif perc <= 0 then
		local r, g, b = ...
		return r, g, b
	end
	
	local num = select('#', ...) / 3

	local segment, relperc = math.modf(perc*(num-1))
	local r1, g1, b1, r2, g2, b2 = select((segment*3)+1, ...)

	return r1 + (r2-r1)*relperc, g1 + (g2-g1)*relperc, b1 + (b2-b1)*relperc
end

hedlib.executes = function()
	local playerClass, englishClass = UnitClass("player")
	if (englishClass=="WARRIOR" or englishClass=="HUNTER" or englishClass=="PALADIN") then
		return 20
	end
	return 0
end

local flashframe = nil
local flashsec = 0
hedlib.flash = function(sec,color)
	color = color or {1, 0, 0, 0.55}
	flashsec=sec
	flashframe = _G["HedFlash"] or CreateFrame("Frame", "HedFlash", UIParent)
	flashframe:SetFrameStrata("BACKGROUND")
	flashframe:SetBackdrop({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",})
	flashframe:SetAllPoints(UIParent)
	flashframe.elapsed = 0
	flashframe:SetScript("OnShow", function(self)
--		self.elapsed = 0
		self:SetAlpha(0)
	end)
	
	flashframe:SetScript("OnUpdate", function(self, elapsed)
		elapsed = self.elapsed + elapsed
		if elapsed >= flashsec then
			self:Hide()
			self:SetAlpha(0)
			self:SetScript("OnUpdate",nil)
			return
		end
		local alpha = elapsed % 0.4
		if elapsed > 0.2 then
			alpha = 0.4 - alpha
		end
		self:SetAlpha(alpha * 5)
		self.elapsed = elapsed
	end)
--	flashframe:Hide()

	flashframe:SetBackdropColor(unpack(color))
	flashframe:Show()
end

--[[ Creating info text ]]
local textframe = CreateFrame("Frame", nil, UIParent)
textframe:SetScript("OnUpdate", FadingFrame_OnUpdate)
textframe.fadeInTime = 0 -- 0.5
textframe.fadeOutTime = 0.5 -- 2
textframe.holdTime = 2 -- 3
textframe:Hide()
hedlib.textframe=textframe

local text = textframe:CreateFontString("HInfoText", "OVERLAY")
text:SetFont("Fonts\\FRIZQT__.TTF", 32, "THICKOUTLINE")
text:SetPoint("CENTER", UIParent, "CENTER",0,350)
text:SetTextColor(0.41, 0.8, 0.94)
hedlib.textframe.text=text

local icon=textframe:CreateTexture(nil, 'OVERLAY')
icon:SetHeight(30)
icon:SetWidth(30)
icon:SetPoint("RIGHT",text,"LEFT")
hedlib.textframe.icon=icon

hedlib.Warning = function(t,i,color)
	if i then
		color = color or {1,0,0}
	else
		color = color or {0.41, 0.8, 0.94}
	end
	hedlib.textframe.text:SetTextColor(unpack(color))
    hedlib.textframe.text:SetText(t)

	if (i) then
		hedlib.textframe.icon:SetTexture(i)
		hedlib.textframe.icon:Show()
	else
		hedlib.textframe.icon:Hide()
	end
	if hedlib.textframe:IsShown() then 
		hedlib.textframe.startTime=GetTime()
	else
	    FadingFrame_Show(hedlib.textframe)
	end
end

hedlib.TargetingYou = function(u)
	local uguid=UnitGUID(u.."target")
	local myguid=UnitGUID("vehicle") or UnitGUID("player")
	if (uguid and uguid==myguid) then
		return true
	else
		return nil
	end
end

hedlib.ClearTable = function(table)
	for k in pairs( table ) do
		table[k] = nil
	end
end

hedlib.tremall = function(tab)
   for k,v in pairs(tab) do
      table.remove(tab, k)
   end
end

hedlib.tremovebyval = function(tab, val)
   for k,v in pairs(tab) do
     if(v==val) then
       table.remove(tab, k)
       return true
     end
   end
   return false
end

hedlib.taddnewval = function(tab, val)
   for k,v in pairs(tab) do
     if(v==val) then
       return false
     end
   end
   table.insert(tab, val)  
   return true
end

local tf,dot
--[[hedlib.tofloat = function(text,position)
	if text then
		if position=="end" then 
			if string.find(text,",")~=nil then
				return tonumber(string.match(text,"%d+,%d+$"))
			end
			tf = string.match(text,"%d+%.%d+$")
			tf = tf or string.match(text,"%d+$")
			tf = tf or 0
			return tonumber(tf)
		end
		if position=="start" then
			if string.find(text,",")~=nil then
				return tonumber(string.match(text,"^%d+,%d+"))
			end
			tf = string.match(text,"^%d+%.%d+")
			tf = tf or string.match(text,"^%d+")
			tf = tf or 0
			return tonumber(tf)
		end
		if string.find(text,",") then
			return tonumber(string.match(text,"%d+,%d+"))
		end
		tf = string.match(text,"%d+%.%d+")
		tf = tf or string.match(text,"%d+")
		tf = tf or 0
		return tonumber(tf)
	end
	return 0
end]]

hedlib.tofloat = function(text,position)
	if text then
		if position=="end" then 
			tf = string.match(text,"%d+%.%d+$")
			tf = tf or string.match(text,"%d+$")
			tf = tf or 0
			return tonumber(tf)
		end
		if position=="start" then
			tf = string.match(text,"^%d+%.%d+")
			tf = tf or string.match(text,"^%d+")
			tf = tf or 0
			return tonumber(tf)
		end
		tf = string.match(text,"%d+%.%d+")
		tf = tf or string.match(text,"%d+")
		tf = tf or 0
		return tonumber(tf)
		
--[[	
		tf = string.match(text,"%d+%.%d+")
		tf = tf or string.match(text,"%d+")
		tf = tf or 0
		return tonumber(tf)]]
	end
	return 0
end

hedlib.classcolor = RAID_CLASS_COLORS[select(2,UnitClass'player')]
hedlib.classcolor = {hedlib.classcolor.r,hedlib.classcolor.g,hedlib.classcolor.b}
hedlib.addbox= function(name, toppoint,bottompoint,color,border,bcolor)
	if color == "CLASS" then
		color = hedlib.classcolor
	end
	if bcolor == "CLASS" then
		bcolor = hedlib.classcolor
	end
	
	local box = _G["HED_"..name] or CreateFrame("Frame","HED_"..name)
	box:SetParent(UIParent)
	box:SetPoint(unpack(toppoint))
	box:SetPoint(unpack(bottompoint))

	box.t = box:CreateTexture(nil, "BACKGROUND")
	box.t:SetAllPoints(box)
	box.t:SetTexture(unpack(color))
		
	if border then
		box:SetBackdrop({
			edgeFile = "Interface/Buttons/WHITE8X8",
			edgeSize = border,
			insets = { 
				left = border, 
				right = border, 
				top = border, 
				bottom = border,
			},
		})
		box:SetBackdropBorderColor(unpack(bcolor))
	end

	box:SetFrameStrata("BACKGROUND")
	box:Show()
end

local copy = {}
hedlib.shallowCopy = function(original)
    copy = {}
    for key, value in pairs(original) do
        copy[key] = value
    end
    return copy
end

hedlib.shallowCompare = function(original,old)
	local same=true
    for key, value in pairs(original) do
		if old[key] and old[key]==value then
			same=true
		else
			same=false
			return false
		end
    end
    return same
end

hedlib.stringifyTable = function(t)
    local entries = {}
    for k, v in pairs(t) do
        -- if we find a nested table, convert that recursively
        if type(v) == 'table' then
            v = hedlib.stringifyTable(v)
        else
            v = tostring(v)
        end
        k = tostring(k)
 
        -- add another entry to our stringified table
        entries[#entries + 1] = ("%s: %s\n"):format(k, v)
    end
 
    -- the memory location of the table
    local id = tostring(t):sub(8)
 
    return ("{%s}@%s"):format(table.concat(entries, ' = '), id)
end

hedlib.print = function(var)
if var then return var end
return "nil"
end

local num
hedlib.largest = function(numbers)
	num=numbers[1]
	for index,number in ipairs(numbers) do
		if num<number then num=number end
	end
	return num
end

hedlib.GetMax = function(numbers)
	num=numbers[1]
	for index,number in ipairs(numbers) do
		if num<number then num=number end
	end
	return num
end

hedlib.GetMin = function(numbers)
	num=numbers[1]
	for index,number in ipairs(numbers) do
		if num<number then num=number end
	end
	return num
end
end